
# q-expansion of newform 4410.2.a.r, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4410
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [0], [1], [0], [2], [0], [4], [-6], [-3], [-9], [-4], [-4], [7], [-5], [-8], [2], [-10], [1], [-9], [-2], [-4], [10], [7], [-1], [14], [-3], [1], [-3], [-9], [-2], [16], [-8], [-12], [-14], [-3], [-16], [10], [-4], [21], [-8], [-12], [7], [18], [26], [-2], [-4], [-26], [28], [4], [22], [-24], [-16], [10], [0], [-8], [-5], [-3], [-6], [12], [-2], [-4], [-28], [7], [18], [8], [32], [-32], [-26], [-19], [-35], [18], [4], [-11], [-4], [30], [-15], [-26], [22], [-31], [3], [0], [-19], [30], [14], [-20], [-31], [33], [-32], [14], [-19], [13], [-24], [-16], [12], [-18], [21], [-1], [-38], [-20], [3], [-33], [2], [-17], [18], [-30], [10], [28], [6], [-12], [42], [1], [12], [-44], [-46], [2], [-5], [28], [11], [4], [26], [-11], [16], [48], [37], [22], [47], [-11], [6], [-21], [22], [-2], [-9], [-4], [16], [6], [-14], [-24], [31], [0], [51], [-28], [18], [19], [19], [-46], [-14], [14], [18], [-48], [11], [38], [7], [-12], [-29], [5], [-30], [38], [-43], [-28], [46], [25], [12], [37], [48], [30], [-17], [40], [-46], [37], [62], [16], [35], [-24], [54], [-4], [-49], [-26], [30], [-49], [43], [-25], [0], [2], [34], [-12], [-45], [40], [-4], [31], [18], [36], [-16], [10], [55], [-12], [-66], [-23], [10], [-42], [-8], [-61], [56], [-6], [45], [0], [-38], [-34], [60], [-9], [-4], [66], [-21], [-47], [-29], [18], [50], [19], [-3], [32], [-72], [17], [14], [39], [24], [7], [23], [-64], [-26], [-41], [4], [40], [66], [58], [-33], [7], [-45], [-41], [4], [-2], [-70], [44], [8], [24], [-11], [0], [6], [77], [60], [-16], [40], [16], [-15], [-33], [-21], [60], [-10], [47], [-11], [42], [-6], [-51], [5], [10], [-28], [-24], [50], [-30], [-14], [-5], [-12], [15], [-72], [-46], [24], [14], [8], [-80], [5], [45], [-22], [-31], [-16], [0], [-21], [-55], [60], [-2], [-72], [-4], [42], [-33], [25], [32], [70], [-54], [81], [22], [14], [6], [-28], [-48], [-38], [-2], [-51], [-4], [72], [-68], [-63], [-24], [-4], [-57], [-54], [-31], [-37], [33], [17], [0], [-54], [-42], [-23], [52], [-32], [7], [51], [-12], [-55], [80], [-27], [32], [-30], [-46], [72], [70], [-3], [-58], [-63], [58], [6], [-35], [8], [-44], [68], [21], [-82], [18], [-58], [45], [-53], [84], [48], [-34], [12], [-9], [17], [45], [0], [-36], [-10], [-24], [33], [2], [34], [-47], [-52], [-11], [52], [56], [40], [37], [-54], [-20], [11], [54], [-88], [52], [-38], [-30], [-4], [90], [-42], [53], [-16], [-33], [72], [0], [-45], [-30], [-4], [-25], [-91], [-82], [-64], [-40], [36], [74], [-2], [-74], [-49], [-12], [51], [15], [-66], [73], [48], [-31], [-62], [68], [33], [3], [48], [50], [18], [12], [-32], [-82], [-106], [18], [24], [42], [-29], [-19], [8], [-93], [-56], [42], [-10], [4], [33], [-32], [46]]
